@@ -21,6 +21,7 @@
 #include <klocale.h>
 #include <kglobal.h>
 #include <kconfiggroup.h>
+#include <knotification.h>
 
 #include "juk.h"
 
@@ -49,7 +50,7 @@ static const char sho[]         = I18N_NOOP("MPRIS2 Interface implementation.");
 int main(int argc, char *argv[])
 {
     KAboutData aboutData("juk", 0, ki18n("JuK"),
-                         "3.10.1", ki18n(description), KAboutData::License_GPL,
+                         "3.11", ki18n(description), KAboutData::License_GPL,
                          ki18n("© 2002–2013, Scott Wheeler, Michael Pyne, and others"),
                          KLocalizedString(),
                          "http://www.kde.org/applications/multimedia/juk/");
@@ -98,8 +99,13 @@ int main(int argc, char *argv[])
         juk->restore(1, false /* don't show */);
 
     KConfigGroup config(KGlobal::config(), "Settings");
-    if(!config.readEntry("StartDocked", false))
+    if(!config.readEntry("StartDocked", false)) {
         juk->show();
+    }
+    else if(!a.isSessionRestored()) {
+        QString message = i18n("JuK running in docked mode\nUse context menu in system tray to restore.");
+        KNotification::event("dock_mode",i18n("JuK Docked"), message);
+    }
 
     return a.exec();
 }
