@@ -114,6 +114,7 @@ PlaylistCollection::PlaylistCollection(PlayerManager *player, QStackedWidget *pl
 PlaylistCollection::~PlaylistCollection()
 {
     saveConfig();
+    CollectionList::instance()->saveItemsToCache();
     delete m_actionHandler;
     Playlist::setShuttingDown();
 }
@@ -512,6 +513,8 @@ void PlaylistCollection::scanFolders()
 
     if(CollectionList::instance()->count() == 0)
         addFolder();
+
+    enableDirWatch(true);
 }
 
 void PlaylistCollection::createPlaylist()
@@ -611,13 +614,13 @@ void PlaylistCollection::setUpcomingPlaylistEnabled(bool enable)
     else {
         action<KToggleAction>("showUpcoming")->setChecked(false);
         bool raiseCollection = visiblePlaylist() == m_upcomingPlaylist;
-        delete m_upcomingPlaylist;
-        m_upcomingPlaylist = 0;
 
         if(raiseCollection) {
-            kapp->processEvents(); // Seems to stop a crash, weird.
             raise(CollectionList::instance());
         }
+
+        m_upcomingPlaylist->deleteLater();
+        m_upcomingPlaylist = 0;
     }
 }
 
